fs          = require "fs"
os          = require "os"
ConfigFile  = require "./config"
utils       = require "./utils"
path        = require "path"
{unique}    = require "underscore"
{logger}    = utils
log         = logger "main"

module.exports = class KodingCLI

  unless fs.existsSync
    fs.existsSync = path.existsSync

  KD_DIR              = path.join process.env.HOME, ".kd"
  MODULE_ROOT         = path.join __dirname, "..", "modules"
  CURRENT_MODULE_ROOT = path.join process.cwd(), ".kd", "modules"
  USER_MODULE_ROOT    = path.join KD_DIR, "modules"

  getModuleList: ->
    if fs.existsSync CURRENT_MODULE_ROOT
      currentAvailable = fs.readdirSync CURRENT_MODULE_ROOT
      currentAvailable = currentAvailable.map (module)->
        path.join CURRENT_MODULE_ROOT, module
    
    if fs.existsSync USER_MODULE_ROOT
      userAvailable = fs.readdirSync USER_MODULE_ROOT
      userAvailable = userAvailable.map (module)->
        path.join USER_MODULE_ROOT, module

    if fs.existsSync MODULE_ROOT
      available = fs.readdirSync MODULE_ROOT
      available = available.map (module)->
        path.join MODULE_ROOT, module
    else
      log "Something wrong! No module found. Did you delete anything?"
      process.exit 1

    available   = available.concat currentAvailable, userAvailable
    available   = available.filter((module)-> module).sort()
    available   = unique available

  constructor: (@module, @command, @params)->

    # root directory of running command is @root
    @root = process.cwd()

    pathParts = @root.split path.sep
    dotKd = pathParts.indexOf ".kd"
    if ~dotKd
      CURRENT_MODULE_ROOT = path.join pathParts[0...dotKd]...
      CURRENT_MODULE_ROOT = path.join path.sep, CURRENT_MODULE_ROOT, ".kd", "modules"

    publicKeyPath = path.join KD_DIR, "koding.key.pub"
    privateKeyPath = path.join KD_DIR, "koding.key"
    try
      publicKey = fs.readFileSync(publicKeyPath).toString()
    catch error
      try
        unless fs.existsSync KD_DIR then fs.mkdirSync KD_DIR
      publicKey = utils.keygen 64
      fs.writeFileSync publicKeyPath, publicKey

    try
      privateKey = fs.readFileSync(privateKeyPath).toString()
    catch error
      try
        unless fs.existsSync KD_DIR then fs.mkdirSync KD_DIR
      privateKey = utils.keygen 64
      fs.writeFileSync privateKeyPath, privateKey

    unless module

      moduleList = @getModuleList()
      console.log """
      Usage:

        kd <module> <command> [arguments]

      The modules are:

      """
      console.log "  " + (path.basename module.replace /.coffee$|.kdmodule$/, '' for module in moduleList when module.match /.coffee$|.kdmodule$/).join ", "
      console.log """

      Use "kd <module> help" for more information about that module.

      """
      return

    @configFile = new ConfigFile

    # Replace module with the alias
    if @configFile.config.alias?[module]
      module = @configFile.config.alias[module]

    @configFile.config.publicKey = publicKey
    @configFile.config.privateKey = privateKey

    # These should show errors inside modules
    # Loading module from the module path.
    lookup = [
      path.join CURRENT_MODULE_ROOT, "#{module}.coffee"
      path.join MODULE_ROOT, "#{module}.coffee"
      path.join USER_MODULE_ROOT, "#{module}.coffee"

      path.join CURRENT_MODULE_ROOT, "#{module}.js"
      path.join MODULE_ROOT, "#{module}.js"
      path.join USER_MODULE_ROOT, "#{module}.js"

      path.join CURRENT_MODULE_ROOT, "#{module}.kdmodule", "index.coffee"
      path.join MODULE_ROOT, "#{module}.kdmodule", "index.coffee"
      path.join USER_MODULE_ROOT, "#{module}.kdmodule", "index.coffee"

      path.join CURRENT_MODULE_ROOT, "#{module}.kdmodule", "index.js"
      path.join MODULE_ROOT, "#{module}.kdmodule", "index.js"
      path.join USER_MODULE_ROOT, "#{module}.kdmodule", "index.js"
      no
    ]

    for modulePath in lookup
      if modulePath then modulePath = path.normalize modulePath
      break if fs.existsSync modulePath

    unless modulePath
      log "Module \"#{module}\" not found.", "red"
      return process.exit 1

    try
      @moduleClass = require modulePath
    catch error
      log error, "red"
      return process.exit 1

    {help, silent, defaultCommand} = @moduleClass.prototype

    unless @command then @command = defaultCommand if defaultCommand

    unless @command
      if help
        console.log "#{help}\n"

      unless help is no
        console.log """

        Usage:

          kd #{module} <command> [arguments]

        The commands of the #{module} module are:

        """

      commands = (command for command, method of @moduleClass.prototype when typeof method is "function" and not command.match /__/).sort()
      console.log "  " + (command for command in commands).join ", "
      console.log ""
      return

    if @command is "help" then return log help

    # Trying to create new instance.
    try
      @moduleClass.utils = require path.join __dirname, "utils.coffee"
      @moduleInstance = new @moduleClass @configFile, @, @getModuleList
    catch error
      unless silent
        log "ERROR: Module instance couldn't be created.", "red"
        log error, "red"
      return process.exit 1

    # Replace command with the alias
    if @moduleInstance.alias?[@command]
      @command = @moduleInstance.alias[@command]

    if @command in ["constructor", "config"]
      log "ERROR: Command #{command} not callable.", "red"
      process.exit 1

    # Trying to *find* new instances method as command
    unless @moduleInstance[@command]
      unless @moduleInstance.__command
        unless silent
          log "ERROR: Command #{command} not found.", "red"
        return process.exit 1
      else
        nocommand = true

    # Trying to run the command.
    try
      @moduleInstance.options = require "optimist"
      @params = @params.filter (param)-> !/^\-+/.test param
      unless nocommand
        @moduleInstance[@command] @params...
      else
        @moduleInstance.__command @command, @params
    catch error
      # If any error occures, show the error.
      unless silent
        log "EXCEPTION: #{error.message or error}", "red"
        return process.exit 1

  # Creating new instance from command line tool.
  @run: (coffeeBin, file, module, command, params...) =>
    new @ module, command, params
