fs    = require "fs"
os    = require "os"
path  = require "path"

{log} = console

module.exports = class Completion

  silent: yes

  KD_BIN = path.normalize path.join __dirname, "..", "bin", "kd"
  CACHE_ROOT = path.join process.env.HOME, ".kd"
  MODULE_ROOT = __dirname
  USER_MODULE_ROOT = path.join process.env.HOME, ".kd", "modules"

  constructor: ({@config}, @main, @getModules)->

  shell: -> log """
  # KD Autocompletion Utility
  # to make completion.

  if type compdef &>/dev/null; then
    _kd_completion() {

      # Defining argument stats
      _arguments \\
        '1: :->_module' \\
        '2: :->_command' \\
        '*: :->_rest' \\
      && return 0

      # State Change
      case $state in
        _module)
          _arguments '1:_module:(`kd completion modules`)'
        ;;
        _command)
          _arguments '2:_command:(`kd completion commands --module=${words[2]}`)'
        ;;
        _rest)
          _files
        ;;
      esac

    }
    compdef _kd_completion kd
  elif type complete &>/dev/null; then
    _kd_completion() {
      if [ "$COMP_CWORD" -ge "3" ]; then
        COMPREPLY=( $(compgen -f -- ${COMP_WORDS[COMP_CWORD]}) )
      else
        COMPREPLY=(`kd completion complete "$COMP_CWORD" "$COMP_LINE"`)
      fi
    }
    complete -F _kd_completion kd
  fi
  """

  modules = (moduleGetter)->
    cacheFile = path.join os.tmpDir(), "koding.kd.completion.modules.json"
    try
      return require cacheFile
    catch error
      moduleList = moduleGetter().map((module)-> path.basename module.replace /\.coffee$|\.kdmodule$/, '')
      fs.writeFileSync cacheFile, JSON.stringify moduleList, null, 2
      moduleList

  commands = (moduleGetter, module)->
    cacheFile = path.join os.tmpDir(), "koding.kd.completion.commands.#{module}.json"
    try
      return require cacheFile
    catch error
      modules = moduleGetter()
      for modulePath in modules
        if (path.basename modulePath) in ["#{module}.js", "#{module}.coffee"]
          moduleClass = require modulePath
        else if (path.basename modulePath) is "#{module}.kdmodule"
          if fs.exists(path.join modulePath, "index.js")
            moduleClass = require path.join modulePath, "index.js"
          else if fs.existsSync(path.join modulePath, "index.coffee")
            moduleClass = require path.join modulePath, "index.coffee"
    try
      commandList = (command for command, method of moduleClass.prototype when typeof method is "function" and not command.match /__/).sort()
      try
        commandList = commandList.concat Object.keys moduleClass.prototype.alias
      fs.writeFileSync cacheFile, JSON.stringify commandList, null, 2
      return commandList

  modules: -> log modules(@getModules).join "\n"
  commands: -> log commands(@getModules, @options.argv.module).join "\n"

  # This is for bash. ZSH is more easy than it. :)
  complete: (cword, line) ->
    cword = parseInt cword
    line = line.split /\s+/
    switch cword
      when 1
        word = line.slice -1
        _modules = modules().filter (module)-> 
          module.match new RegExp("^#{word}")
        log _modules.join "\n"
      when 2
        module = line.slice -2, -1
        word = line.slice -1
        _commands = commands(module).filter (command)-> 
          command.match new RegExp("^#{word}")
        log _commands.join "\n"

  install: ->
    # Because of OS X's EPIPE error on bash, we need to give
    # executable permission and run.
    log """
    #{KD_BIN} completion shell > ~/.kd/completion.sh 
    chmod +x ~/.kd/completion.sh 
    echo "source ~/.kd/completion.sh # KD" >> ~/.profile
    """