fs      = require "fs"
os      = require "os"
colors  = require "colors"
path    = require "path"
http    = require "http"
request = require "request"
qs      = require "querystring"
utils   = require "../lib/utils"

{ask, logger} = utils
log     = logger "register"

module.exports = class Register

  alias:
    link: "computer"

  defaultCommand: "--dev"

  KD_DIR = path.join process.env.HOME, ".kd"

  constructor: (@configuration)->

    {@config} = @configuration

    @privateKeyPath = path.join KD_DIR, "koding.key"
    @publicKeyPath = path.join KD_DIR, "koding.key.pub"

    @config.KODING_ROOT or= "http://koding.com"

  # REFACTOR HERE
  "--dev": (root="http://new.koding.com")->
    Config = require "./config"
    config = new Config @configuration

    if @options.argv.dev
      log (colors.rainbow """
        
        Switching to the develop mode.
        
        """), "magenta"
      config.set "KODING_ROOT", root
    else
      config.remove "KODING_ROOT"
      @config.KODING_ROOT = "http://koding.com"

    unless @config["user.name"]
      config.username =>
        @config.publicKey = fs.readFileSync(@publicKeyPath).toString().trim()
        @computer()
      , yes # yes means don't exit after quit, i'll manage it.
    else
      @config.publicKey = fs.readFileSync(@publicKeyPath).toString().trim()
      @computer()

  get: ->
    console.log @config.publicKey

  renew: ->

    {argv: {all, silent}} = @options

    change = (path, name)=>
      key = utils.keygen 64
      fs.writeFileSync path, key
      console.log "#{name}: #{key}"

    unless silent
      ask "Do you really want to do it? It may affect your works! (yes|no)", 
        format: /yes|no|Y|N/,
        callback: (answer)=>
          if answer is "yes"
            change @publicKeyPath, "Public"
            if all then change @privateKeyPath, "Private"
          process.exit()
        error: (answer)->
          console.log "Please write a valid answer: yes or no. That's simple."
    else
      change @publicKeyPath, "Public"
      if all then change @privateKeyPath, "Private"

  where: ->
    console.log @publicKeyPath

  set: (key)->
    unless key?.length is 64
      log "Your key's length must be 64.", "red"
      process.exit 1
    fs.writeFileSync (path.join process.env.HOME, ".kd", "koding.key.pub"), key
    log "You've changed your key successfully.", "green"

  computer: ->

    {argv: {verbose}} = @options
      .alias("v", "verbose")
      .alias("t", "timeout")

    link = "#{@config.KODING_ROOT}/-/KD/Register/#{encodeURIComponent os.hostname()}/#{@config.publicKey}"
    log """


    Hello #{colors.blue @config["user.name"]}!

    Please visit the following URL to connect your 
    host to this computer.


    """
    
    log """
    #{colors.underline link}

    """, "green"
    check = null
    process.stdout.write colors.blue "register: "
    process.stdout.write "Waiting for you to connect the host"
    waiting = setInterval =>
      check?.abort?()
      check = request.post "#{@config.KODING_ROOT}/-/kd/register-check",
        form: 
          key: @config.publicKey
          username : @config["user.name"]
      , (err, res)=>
        if err or not res
          process.stdout.write os.EOL
          log "Couldn't connect to Koding. Check your internet connection.", "red"
          log err.message.toString(), "red"  if verbose
          clearInterval waiting
          process.exit 1
        if res.statusCode isnt 200
          unless res.statusCode is 418
            process.stdout.write os.EOL
            log "An error occured with the status #{res.statusCode}: #{http.STATUS_CODES[res.statusCode]}.", "red"
            clearInterval waiting
            process.exit 1
        else
          process.stdout.write "\n"
          if res.body is "OK"
            log "Host connected! Thanks for registering, now I'm sure you are #{@config["user.name"]}!", "green"
          else
            log "Registration has failed. Check your internet connection.", "red"
            log (colors.inverse "Are you sure #{@config.KODING_ROOT} is up?"), "magenta"  if @options.argv.dev
            log (colors.inverse "This is what I got: #{res.body}"), "magenta" if @options.argv.dev
          clearInterval waiting
          process.exit 0
      process.stdout.write "."
    , 1000
    setTimeout ->
      process.stdout.write os.EOL
      log "Registration timed out. Please try again.", "red"
      clearInterval waiting
      process.exit 1
    , (@options.argv.timeout || 30)*1000